'''Модуль обработки токенов'''
import requests
import exeptions

class KeycloackToken():
    '''Класс для обработки токенов'''
    client_name = ""
    client_secret = ""
    api_url = ""

    def refresh_token(self, access_token: str, refresh_token: str) -> dict:
        '''Обновление токена'''
        data = {
            'client_id': self.client_name,
            'client_secret': self.client_secret,
            'accesss_token': access_token,
            'refresh_token': refresh_token,
            'grant_type': 'refresh_token'
        }
        response = requests.post(f'{self.api_url}/token', data=data, timeout=3)
        exeptions.check_status_code(response.status_code)
        return response.json()

    def revoke_token(self, access_token: str) -> None:
        '''Отозвать токен'''
        data = {
            'client_id': self.client_name,
            'client_secret': self.client_secret,
            'token': access_token,
            'token_type_hint': 'access_token'
        }
        response = requests.post(f'{self.api_url}/revoke', data=data, timeout=3)
        exeptions.check_status_code(response.status_code)

    def introspect_token(self, access_token: str) -> dict:
        '''Получение интроспекции токена'''
        data = {
            'client_id': self.client_name,
            'client_secret': self.client_secret,
            'token': access_token
        }
        response = requests.post(f'{self.api_url}/token/introspect', data=data, timeout=3)
        exeptions.check_status_code(response.status_code)
        return response.json()



'''Модуль обработки ролей'''
import json

import requests
import exeptions

class KeycloackRole():
    '''Класс для работы с ролями'''
    client_name = ""
    client_secret = ""
    admin_url = ""
    client_id = ""

    def get_roles(self, access_token: str) -> list or Exception:
        """
        Получение ролей
        Args:
            id_client (str): id клиента
            access_token (str): токен администратора

        Returns:
            list or Exception: _description_
        """
        url = f"{self.admin_url}/clients/{self.client_id}/roles"
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(url, headers=headers)
        exeptions.check_status_code(response.status_code)
        return response.json()

    def assign_role_to_user(self, user_id: str, role: dict, access_token: str):
        """
        Добавление роли пользователю
        Args:
            user_id (str): user id
            role (dict): роль
            id_client (str): ид клиента
            access_token (str): токен администратора
        """

        url = f"{self.admin_url}/users/{user_id}/role-mappings/clients/{self.client_id}"
        headers = {
            "Accept": "application/json",
            'Content-Type': 'application/json',
            "Authorization": f"bearer {access_token}"
        }
        data = json.dumps([role])
        response = requests.post(url, headers=headers, data=data)
        exeptions.check_status_code(response.status_code)



'''Модуль обработки токенов'''
import requests
import exeptions

class KeycloackUsers():
    '''Класс для обработки токенов'''
    client_name = ""
    client_secret = ""
    admin_url = ""

    def get_all_users(self, access_token: str):
        """
        Получение всех пользователей
        Args:
            access_token (str): токен администратора

        Returns:
            List[Dict[str, Any]]: Список пользователей
        """
        headers = {
            "Accept": "application/json",
            "Authorization": f"bearer {access_token}"
        }
        response = requests.get(
            f'{self.admin_url}/users',
            headers=headers, timeout=3)
        exeptions.check_status_code(response.status_code)
        users = response.json()
        return users



"""Кастомные ошибки"""
class UserAlreadyExists:
    """Ошибка возникающая при попытке зарегестрировать с существующим username"""
    pass

class Unauthorized:
    """Ошибка возникающая при status_code==401"""
    pass

class PageNotFound:
    """Ошибка возникающая при некорректном url для keycloak"""
    pass

class BadRequest:
    """Ошибка возникающая при некорректном данных"""
    pass

def check_status_code(status_code):
    '''Проверка status code'''
    if status_code == 401:
        raise Unauthorized('Incorrect login/password')
    elif status_code == 404:
        raise PageNotFound('Incorrect API keyckloak')
    elif status_code == 400:
        raise BadRequest("Invalid credentionals")
    elif status_code == 409:
        raise UserAlreadyExists("User allredy exists")
    elif status_code == 403:
        raise Unauthorized('Method not allowed')



import json
import requests
import keycloak_srvcloud.exeptions as exeptions
from .keycloak_roles import KeycloackRole
from .keycloak_tokens import KeycloackToken
from .keycloak_users import KeycloackUsers

class Keycloak(KeycloackRole, KeycloackToken, KeycloackUsers):
    '''Класс для работы с keycloak'''
    keycloak_host = ""
    keycloak_port = ""
    client_name = ""
    client_id = ""
    client_secret = ""
    admin_user = ""
    admin_password = ""
    api_url = ""
    admin_url = ""

    def __init__(self,
                 keycloak_host: str,
                 keycloak_port: str,
                 realm_name: str,
                 client_name: str,
                 client_id: str,
                 client_secret: str,
                 admin_user: str,
                 admin_password: str) -> None:
        """
        Инициализация класса
        Args:
            keycloak_host (str): ip keycloak
            keycloak_port (str): port keycloak
            realm_name (str): Название realm
            client_id (str): id клиента
            client_secret (str): secret клиента
            admin_user (str): username admin user
            admin_password (str): password admin user
        """
        self.keycloak_host = keycloak_host
        self.keycloak_port = keycloak_port
        self.realm_name = realm_name
        self.client_name = client_name
        self.client_id = client_id
        self.client_secret = client_secret
        self.admin_user = admin_user
        self.admin_password = admin_password
        self.api_url = f"http://{keycloak_host}:{keycloak_port}/realms/{realm_name}/protocol/openid-connect"
        self.admin_url = f"http://{keycloak_host}:{keycloak_port}/admin/realms/{realm_name}"

    def login(self, username: str, password: str) -> dict:
        '''Авторизация в keycloak'''
        data = {
            'client_id': self.client_name,
            'client_secret': self.client_secret,
            'username': username,
            'password': password,
            'grant_type': "password"
        }
        response = requests.post(f'{self.api_url}/token', data=data, timeout=3)
        exeptions.check_status_code(response.status_code)
        return response.json()

    def logout(self, access_token: str, refresh_token: str) -> None:
        '''Завершает сессию'''
        headers = {
            'Authorization': f"Bearer {access_token}",
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        data = {
            'client_id': self.client_name,
            'client_secret': self.client_secret,
            'refresh_token': refresh_token
        }
        response = requests.post(f'{self.api_url}/logout', data=data, headers=headers, timeout=3)
        exeptions.check_status_code(response.status_code)


    def userinfo(self, access_token: str) -> dict:
        '''Получение userinfo'''
        headers = {
            'Authorization': f"Bearer {access_token}",
            'Content-Type': 'application/json'
        }
        response = requests.get(f'{self.api_url}/userinfo', headers=headers, timeout=3)
        exeptions.check_status_code(response.status_code)
        return response.json()


    def sign_up(self, access_token: str, username: str, password: str,
                first_name: str = "", last_name: str = "", email="") -> None or Exception:
        """
        Регистрация пользователя
        Args:
            access_token (str): токен админстраторв
            username (str): логин
            password (str): пароль
            first_name (str, optional): Имя. Defaults to "".
            last_name (str, optional): Фамилия. Defaults to "".

        Returns:
            None or Exception: При успешной регистрации ничего не возвращается
        """
        headers = {
            "Accept": "application/json",
            'Content-Type': 'application/json',
            "Authorization": f"bearer {access_token}"
        }
        data = json.dumps({
            "email": email,
            "emailVerified": False,
            "enabled": True,
            "firstName": first_name,
            "groups": [],
            "lastName": last_name,
            "requiredActions": [],
            "username": username,
            "credentials": [{"type": "password", "value": password, "temporary": False}]
        })
        response = requests.post(
            f'http://{self.keycloak_host}:{self.keycloak_port}/admin/realms/{self.realm_name}/users',
            data=data, headers=headers, timeout=3)
        exeptions.check_status_code(response.status_code)

